package compbio.stat.collector;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;

import compbio.engine.conf.PropertyHelperManager;
import compbio.util.Util;
import compbio.ws.client.Services;

/**
 * The database must be stored in the application root directory and called
 * "ExecutionStatistic"
 * 
 * @author pvtroshin
 * 
 */
public class StatDB {

	private static final String driver = "org.apache.derby.jdbc.EmbeddedDriver";
	private static final String protocol = "jdbc:derby:";
	private static final String statDBName = "ExecutionStatistic";

	private static final Logger log = Logger.getLogger(StatDB.class);

	Connection conn;
	private static Connection getDBConnection() throws SQLException {
		String dbpath = PropertyHelperManager.getLocalPath();
		log.info("Looking for JABAWS access statistics database at: " + dbpath);
		System.setProperty("derby.system.home", dbpath);
		Connection conn = DriverManager.getConnection(protocol + statDBName
				+ ";create=false");

		conn.setAutoCommit(true);
		return conn;
	}

	public StatDB() throws SQLException {
		this.conn = getDBConnection();
	}

	/**
	 * Connect to test database
	 * 
	 * @param ignored
	 * @throws SQLException
	 */
	StatDB(boolean ignored) throws SQLException {
		this.conn = getTestDBConnection();
	}

	private static Connection getTestDBConnection() throws SQLException {
		System.setProperty("derby.system.home", "testsrc/testdata");
		Connection conn = DriverManager.getConnection(protocol + statDBName
				+ ";create=false");
		conn.setAutoCommit(true);
		return conn;
	}

	// ServiceName,jobname,start,finish,inputSize,resultSize,isCancelled,isCollected
	/**
	 * 
	 * rs.getBoolean(i) will return true for any non-zero value and false for 0
	 * on SMALLINT data column.
	 * 
	 * @throws SQLException
	 */
	private void createStatTable() throws SQLException {

		/*
		 * Creating a statement object that we can use for running various SQL
		 * statements commands against the database.
		 */
		Statement s = conn.createStatement();
		String create = "create table exec_stat("
				+ "number INT GENERATED ALWAYS AS IDENTITY,"
				+ "service_name VARCHAR(15) NOT NULL, "
				+ "cluster_job_id VARCHAR(30), "
				+ "job_id VARCHAR(35) NOT NULL PRIMARY KEY, "
				+ "start TIMESTAMP," + "finish TIMESTAMP,"
				+ "inputsize BIGINT," + "resultsize BIGINT,"
				+ "isCancelled SMALLINT NOT NULL,"
				+ "isCollected SMALLINT NOT NULL, "
				+ "isClusterJob SMALLINT NOT NULL)";
		// We create a table...
		System.out.println(create);
		s.execute(create);
		s.close();
		conn.close();
	}

	void insertData(Set<JobStat> jobstatus) throws SQLException {
		System.out.println("Inserting " + jobstatus.size());

		conn.setAutoCommit(false);
		String insert = "insert into exec_stat (service_name, cluster_job_id, job_id, start, finish, "
				+ "inputsize, resultsize, isCancelled, isCollected, isClusterJob) "
				+ "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
		PreparedStatement pstm = conn.prepareStatement(insert);
		for (JobStat js : jobstatus) {
			// Has to present
			pstm.setString(1, js.webService.toString());

			if (!Util.isEmpty(js.clusterJobId)) {
				pstm.setString(2, js.clusterJobId);
			} else {
				pstm.setString(2, null);
			}
			// Has to present
			pstm.setString(3, js.jobname);

			if (js.start != ExecutionStatCollector.UNDEFINED) {
				pstm.setTimestamp(4, new Timestamp(js.start));
			} else {
				pstm.setTimestamp(4, null);
			}
			if (js.finish != ExecutionStatCollector.UNDEFINED) {
				pstm.setTimestamp(5, new Timestamp(js.finish));
			} else {
				pstm.setTimestamp(5, null);
			}
			// -1 if UNDEFINED
			pstm.setLong(6, js.inputSize);
			// -1 if UNDEFINED
			pstm.setLong(7, js.resultSize);

			pstm.setBoolean(8, js.isCancelled);
			pstm.setBoolean(9, js.isCollected);
			pstm.setBoolean(10, js.isClusterJob());
			pstm.executeUpdate();
		}
		conn.commit();
		pstm.close();
	}

	public List<JobStat> readData(Timestamp from, Timestamp to,
			Services wservice, Boolean clusterOnly) throws SQLException {

		String query = "select service_name, cluster_job_id, job_id, start, finish, inputsize, "
				+ "resultsize, isCancelled, isCollected from exec_stat where start BETWEEN ? and ? ";

		if (wservice != null) {
			query += " and service_name=? ";
		}

		if (clusterOnly != null) {
			if (clusterOnly) {
				query += " and isClusterJob!=0 ";
			} else {
				query += " and isClusterJob=0 ";
			}
		}

		PreparedStatement pstm = conn.prepareStatement(query);
		pstm.setTimestamp(1, from);
		pstm.setTimestamp(2, to);
		if (wservice != null) {
			pstm.setString(3, wservice.toString());
		}
		pstm.execute();
		List<JobStat> stats = new ArrayList<JobStat>();
		ResultSet rs = pstm.getResultSet();
		while (rs.next()) {
			stats.add(JobStat.newInstance(Services.getService(rs.getString(1)),
					rs.getString(2), rs.getString(3), rs.getTimestamp(4),
					rs.getTimestamp(5), rs.getLong(6), rs.getLong(7),
					rs.getBoolean(8), rs.getBoolean(9)));
		}
		rs.close();
		pstm.close();

		return stats;
	}
	public void removeRecordedJobs(Set<JobStat> fsJobs) throws SQLException {

		String query = "select job_id from exec_stat";

		Statement st = conn.createStatement();
		ResultSet result = st.executeQuery(query);

		while (result.next()) {
			String recordedJob = result.getString(1);
			JobStat recStat = JobStat.newIncompleteStat(recordedJob);
			if (fsJobs.contains(recStat)) {
				fsJobs.remove(recStat);
			}
		}
		result.close();
	}

	public void shutdownDBServer() {
		// ## DATABASE SHUTDOWN SECTION ##
		/***
		 * In embedded mode, an application should shut down Derby. Shutdown
		 * throws the XJ015 exception to confirm success.
		 ***/
		try {
			if (conn != null) {
				conn.close();
			}
		} catch (SQLException e) {
			System.err.println("Database commit failed with "
					+ e.getLocalizedMessage());
		}
		boolean gotSQLExc = false;
		try {
			DriverManager.getConnection("jdbc:derby:;shutdown=true");
		} catch (SQLException se) {
			if (se.getSQLState().equals("XJ015")) {
				gotSQLExc = true;
			}
		}
		if (!gotSQLExc) {
			System.err.println("Database did not shut down normally");
		} else {
			System.out.println("Database shut down normally");
		}
	}
	public static void main(String[] args) throws SQLException {
		// new StatDB().createStatTable();
		// insertData(null);
		/*
		 * StatDB statdb = new StatDB(); Date from = new Date();
		 * from.setMonth(1); System.out.println(new
		 * StatProcessor(statdb.readData( new Timestamp(from.getTime()), new
		 * Timestamp(new Date().getTime()), null, null)).reportStat());
		 */
	}
}
